/* $Id: globus_i_replica_management.c,v 1.9 2001/04/03 19:16:28 meder Exp $ */

#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file
 *
 * Globus Replica Management Internal C API Source File
 */
#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */

#include "globus_replica_management.h"
#include "globus_i_replica_management.h"

globus_result_t
globus_i_replica_management_monitor_init(
    globus_i_replica_management_monitor_t *     monitor)
{
    globus_result_t                     rc;
    static char * myname = "globus_i_replica_management_monitor_init";
    
    if(monitor == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_MANAGEMENT_ERROR(
	    "NULL monitor",
	    myname);
	goto return_result;
    }

    globus_mutex_init(&monitor->mutex, GLOBUS_NULL);

    globus_cond_init(&monitor->cond, GLOBUS_NULL);

    monitor->done = GLOBUS_FALSE;
    
    monitor->result = GLOBUS_SUCCESS;

    monitor->bytes_read = 0;
    
    rc = GLOBUS_SUCCESS;
    
 return_result:
    return rc;
}

globus_result_t
globus_i_replica_management_monitor_reset(
    globus_i_replica_management_monitor_t *     monitor)
{
    globus_result_t                     rc;
    static char * myname = "globus_i_replica_management_monitor_reset";
    
    if(monitor == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_MANAGEMENT_ERROR(
	    "NULL monitor",
	    myname);
	goto return_result;
    }

    monitor->done = GLOBUS_FALSE;
    
    monitor->result = GLOBUS_SUCCESS;

    monitor->bytes_read = 0;
    
    rc = GLOBUS_SUCCESS;
    
 return_result:
    return rc;
}

globus_result_t
globus_i_replica_management_monitor_destroy(
    globus_i_replica_management_monitor_t *     monitor)
{
    globus_result_t                     rc;
    static char * myname = "globus_i_replica_management_monitor_destroy";
    
    if(monitor == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_MANAGEMENT_ERROR(
	    "NULL monitor",
	    myname);
	goto return_result;
    }

    globus_mutex_destroy(&monitor->mutex);

    globus_cond_destroy(&monitor->cond);

    monitor->done = GLOBUS_FALSE;

    monitor->result = GLOBUS_SUCCESS;

    monitor->bytes_read = 0;
    
    rc = GLOBUS_SUCCESS;
    
 return_result:
    return rc;
}


globus_result_t
globus_i_replica_management_set_plugin_data(
    globus_replica_management_handle_t *    handle,
    char *                                  location_name,
    char *                                  filename)
{
    globus_result_t                     rc;
    static char * myname = "globus_i_replica_management_set_plugin_data";
    
    if(handle == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_MANAGEMENT_ERROR(
	    "NULL handle",
	    myname);
	goto return_result;
    }

    if(location_name == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_MANAGEMENT_ERROR(
	    "NULL location name",
	    myname);
	goto return_result;
    }
    
    if(filename == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_MANAGEMENT_ERROR(
	    "NULL file name",
	    myname);
	goto return_result;
    }

    handle->plugin_data.location_name = location_name;

    handle->plugin_data.filename = filename;

    handle->plugin_data.update_count = 0;
    
    handle->plugin_data.final_response_count = 0;

    rc = GLOBUS_SUCCESS;
    
 return_result:
    return rc;
}

globus_result_t
globus_i_replica_management_plugin_data_init(
    globus_replica_management_handle_t *    handle)
{
    globus_result_t                     rc;
    static char * myname = "globus_i_replica_management_plugin_data_init";
    
    if(handle == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_MANAGEMENT_ERROR(
	    "NULL handle",
	    myname);
	goto return_result;
    }

    handle->plugin_data.handle = handle;
    
    handle->plugin_data.location_name = GLOBUS_NULL;

    handle->plugin_data.filename = GLOBUS_NULL;

    handle->plugin_data.update_count = 0;

    handle->plugin_data.final_response_count = 0;
    
    rc = GLOBUS_SUCCESS;
    
 return_result:
    return rc;
}





