#include "globus_replica_management_progress_plugin.h"
#include "globus_i_replica_catalog.h"
#include <ctype.h>
#include <stdio.h>


static
globus_ftp_client_plugin_t *
globus_l_replica_management_progress_plugin_copy(
    globus_ftp_client_plugin_t *		plugin,
    void *					plugin_specific)
{
    return plugin;
}

static
void
globus_l_replica_management_progress_plugin_destroy(
    globus_ftp_client_plugin_t *		plugin,
    void *					plugin_specific)
{
    return;
}

static
void 
globus_l_replica_management_progress_plugin_transfer(
    globus_ftp_client_plugin_t *		plugin,
    void *					plugin_specific,
    globus_ftp_client_handle_t *		handle,
    const char *				source_url,
    const globus_ftp_client_operationattr_t *	source_attr,
    const char *				dest_url,
    const globus_ftp_client_operationattr_t *	dest_attr,
    globus_bool_t				restart)
{
    globus_replica_management_plugin_data_t *   data;
    globus_result_t                             rc;
    
    data = (globus_replica_management_plugin_data_t *) plugin_specific;
    
    /* initialize the restart marker */

    if(restart == GLOBUS_TRUE)
    {
	rc = globus_ftp_client_restart_marker_destroy(&data->marker);

	if(rc != GLOBUS_SUCCESS)
	{
	    globus_ftp_client_plugin_abort(handle);
	    return;
	}
    }

    rc = globus_ftp_client_restart_marker_init(&data->marker);

    if(rc != GLOBUS_SUCCESS)
    {
	globus_ftp_client_plugin_abort(handle);
	return;
    }
    
    return;
}

static
void
globus_l_replica_management_progress_plugin_response(
    globus_ftp_client_plugin_t *		plugin,
    void *					plugin_specific,
    globus_ftp_client_handle_t *		handle,
    const char *				url,
    globus_object_t *				error,
    const globus_ftp_control_response_t *	ftp_response)
{
    globus_replica_management_plugin_data_t *   data;
    globus_result_t                             rc;
    unsigned int                                frequency;
    globus_off_t				offset;
    globus_off_t                                end;
    char *					p;
    char *					marker_string;
    int						consumed;

    if(error != GLOBUS_SUCCESS)
    {
	return;
    }

    
    data = (globus_replica_management_plugin_data_t *) plugin_specific;

    
    /* if response code is xxx add range to restart marker
       and update catalog. Might need attribute for how often to
       update
    */


    switch(ftp_response->code)
    {
    case 111:
	/* restart marker */
	data->update_count++;
	rc = globus_replica_management_handleattr_get_ftp_update_frequency(
	    &data->handle->attrs,
	    &frequency);
	    
	if(rc != GLOBUS_SUCCESS)
	{
	    globus_ftp_client_plugin_abort(handle);
	    return;
	}
	    
	if(data->update_count%frequency == 0)
	{
	    p = (char *) ftp_response->response_buffer;
		
	    /* skip 111 Range Marker */
	    p += 4;
		
	    while(!isdigit(*p)) p++;
		
	    while( sscanf(p, "%"GLOBUS_OFF_T_FORMAT"-%"GLOBUS_OFF_T_FORMAT"%n",
			  &offset, &end, &consumed) >= 2)
	    {
		rc = globus_ftp_client_restart_marker_insert_range(
		    &data->marker,
		    offset,
		    end);
		    
		if(rc != GLOBUS_SUCCESS)
		{
		    rc = globus_ftp_client_plugin_abort(handle);
		    globus_assert(rc == GLOBUS_SUCCESS);
		    return;		
		}
		    
		p += consumed;
		    
		if(*p == ',')
		{
		    p++;
		}
		else
		{
		    break;
		}
	    }
	    
	    rc = globus_ftp_client_restart_marker_to_string(
		&data->marker,
		&marker_string);
	    
	    if(rc != GLOBUS_SUCCESS)
	    {
		rc = globus_ftp_client_plugin_abort(handle);
		globus_assert(rc == GLOBUS_SUCCESS);
		return;
	    }
		    
	    rc = GLOBUS_I_REPLICA_CATALOG_ROLLBACK_INFO_UPDATE(
		data->handle->replica_handle,
		data->location_name,
		data->filename,
		GLOBUS_NULL,
		marker_string);

	    globus_libc_free(marker_string);
	    
	    if(rc != GLOBUS_SUCCESS)
	    {
		rc = globus_ftp_client_plugin_abort(handle);
		globus_assert(rc == GLOBUS_SUCCESS);
		return;
	    }
	}
	break;
    case 226:
    case 250:
	/* transfer successfully completed */

	data->update_count = 0;
	data->final_response_count++;

	if(data->final_response_count == 2)
	{
	    data->final_response_count = 0;
	    
	    rc = GLOBUS_I_REPLICA_CATALOG_ROLLBACK_INFO_UPDATE(
		data->handle->replica_handle,
		data->location_name,
		data->filename,
		GLOBUS_NULL,
		"done");
	    
	    if(rc != GLOBUS_SUCCESS)
	    {
		rc = globus_ftp_client_plugin_abort(handle);
		globus_assert(rc == GLOBUS_SUCCESS);
		return;
	    }
	}
	break;
    default:
	break;
    }
    
    return;
}

static
void
globus_l_replica_management_progress_plugin_complete(
    globus_ftp_client_plugin_t *		plugin,
    void *					plugin_specific,
    globus_ftp_client_handle_t *		handle)
{
    globus_replica_management_plugin_data_t *   data;
    globus_result_t                             rc;

    data = (globus_replica_management_plugin_data_t *) plugin_specific;
    
    rc = globus_ftp_client_restart_marker_destroy(&data->marker);
    
    if(rc != GLOBUS_SUCCESS)
    {
	globus_ftp_client_plugin_abort(handle);
    }

    return;
}

globus_result_t
globus_i_replica_management_progress_plugin_init(
    globus_ftp_client_plugin_t *			plugin,
    globus_replica_management_plugin_data_t *           data)
{
    globus_result_t					result;
    globus_object_t *					err;

    result = globus_ftp_client_plugin_init(
	    plugin,
	    "globus_replica_management_progress_plugin",
	    GLOBUS_FTP_CLIENT_CMD_MASK_FILE_ACTIONS,
	    data);
    if(result != GLOBUS_SUCCESS)
    {
	return result;
    }
    
    result = globus_ftp_client_plugin_set_copy_func(
	    plugin,
	    globus_l_replica_management_progress_plugin_copy);
    if(result != GLOBUS_SUCCESS)
    {
	goto error_exit;
    }

    result = globus_ftp_client_plugin_set_destroy_func(
	    plugin,
	    globus_l_replica_management_progress_plugin_destroy);
    if(result != GLOBUS_SUCCESS)
    {
	goto error_exit;
    }

    result = globus_ftp_client_plugin_set_third_party_transfer_func(
	    plugin,
	    globus_l_replica_management_progress_plugin_transfer);
    if(result != GLOBUS_SUCCESS)
    {
	goto error_exit;
    }

    result = globus_ftp_client_plugin_set_response_func(
	    plugin,
	    globus_l_replica_management_progress_plugin_response);
    if(result != GLOBUS_SUCCESS)
    {
	goto error_exit;
    }

    result = globus_ftp_client_plugin_set_complete_func(
	    plugin,
	    globus_l_replica_management_progress_plugin_complete);
    if(result != GLOBUS_SUCCESS)
    {
	goto error_exit;
    }

    return GLOBUS_SUCCESS;
error_exit:
    err = globus_error_get(result);
    globus_ftp_client_plugin_destroy(plugin);
    return globus_error_put(err);
}

globus_result_t
globus_i_replica_management_progress_plugin_destroy(
    globus_ftp_client_plugin_t *			plugin)
{
    return GLOBUS_SUCCESS;
}
